local E, L = unpack(select(2, ...)) -- Engine, Locale
local CO, UF = E:LoadModules("Config", "Unitframes")

--[[--------------------
	Unitframe Extension	
--------------------]]--

local _
local tinsert 			= table.insert
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local UnitPower 		= UnitPower
local UnitPowerMax 		= UnitPowerMax
local Module = {}

-----------------------------------------

Module.Frames = {}

local function UpdateElement(Element, Event, Unit)
	if Element.Disabled then return end
	if not Unit then Unit = Element.Unit end -- Required for ForceUpdate
	
	if Event == "UNIT_DISPLAYPOWER" or Event == "ForceUpdate" then
		UF:UpdateBarColor(Element, E:GetUnitPowerColor(Unit))
	end
	
	if Event == "UNIT_MAXPOWER" or Event == "UNIT_DISPLAYPOWER" or Event == "UNIT_POWER_BAR_SHOW" or Event == "UNIT_POWER_BAR_HIDE"
	or Event == "ForceUpdate" then
		Element.MaxValue = UnitPowerMax(Unit)
		
		if Element.MaxValue == 0 then
			if Element.Visible then
				Element:Hide()
				Element.Visible = nil
			end
			
			return
		else
			if not Element.Visible then
				Element:Show()
				Element.Visible = true
			end
			
			Element:SetMinMaxValues(0, Element.MaxValue)
		end
	end
	
	if not UnitIsDeadOrGhost(Unit) then
		Element:SetValue(UnitPower(Unit))
	else
		Element:SetValue(0)
	end
end

local function ForceUpdate(Element)
	UpdateElement(Element, "ForceUpdate", Element.Unit)
end

local function OnEvent(Element, event, unit)
	if(not unit or Element.Unit ~= unit) then return end
	UpdateElement(Element, event, unit)
end

----------

function Module:LoadProfile()
	local Config, Element
	
	for _, F in pairs(Module.Frames) do
		Config = self.db.units[F.ProfileUnit].power
		Element = F.Power
		
		Element:SetSize(Config.barWidth, Config.barHeight)
		Element:SetReverseFill(Config.barInverseFill)
		Element:SetOrientation(Config.barOrientation)
		if Config.barSmooth then
			E.Libs.LibSmooth:SmoothBar(Element)
		else
			E.Libs.LibSmooth:ResetBar(Element)
		end

		Element:ClearAllPoints()
		Element:SetParent(F.Overlay)
		Element:SetPoint(Config.barPosition, F.Health, Config.barPosition, Config.barXOffset, Config.barYOffset)
		
		-- Texture
		if Config.overrideBarTexture then
			Element:SetAttribute("ReceivesGlobalTexture", false)
			Element:SetStatusBarTexture(E.Media:Fetch("statusbar", Config.barTexture or self.db.units.all.barTexture))
		else
			Element:SetAttribute("ReceivesGlobalTexture", true)
			Element:SetStatusBarTexture(E.Media:Fetch("statusbar", self.db.units.all.barTexture))
		end
		
		-- Power Border and Background
		Element.Background:SetColorTexture(unpack(Config.barBackgroundColor))
		E:SetFrameBorder(Element.Border, Config.barBorderSize, unpack(Config.barBorderColor))
		
		if not F.Eventless then
			if Config.fastUpdate then
				Element:UnregisterEvent("UNIT_POWER_UPDATE")
				Element:RegisterEvent("UNIT_POWER_FREQUENT")
			else
				Element:UnregisterEvent("UNIT_POWER_FREQUENT")
				Element:RegisterEvent("UNIT_POWER_UPDATE")
			end
			
			if not Element:IsEventRegistered("UNIT_MAXPOWER") then
				Element:RegisterEvent("UNIT_MAXPOWER")
				Element:RegisterEvent("UNIT_DISPLAYPOWER")
				Element:RegisterEvent("UNIT_POWER_BAR_SHOW")
				Element:RegisterEvent("UNIT_POWER_BAR_HIDE")
			end
		end
		
		Element.Disabled = false
	end
end

function Module:Create(F)
	local Element 	= UF:CreateUFBar()
	F.Power 		= Element
	
	Element.Border 		= E:CreateBorder(Element)
	Element.Background 	= E:CreateBackground(Element)
	
	Element.Visible = true
	
	Element.Unit = F.Unit
	Element:SetScript("OnEvent", OnEvent)
	Element.ForceUpdate = ForceUpdate
	
	tinsert(Module.Frames, F)
end

---------- Add Module
UF:RegisterModule('BarPower', Module)