local E, L = unpack(select(2, ...)) -- Engine, Locale
local CO, UF = E:LoadModules("Config", "Unitframes")

--[[--------------------
	Unitframe Extension	
--------------------]]--

local _
local UpdateInterval = 0.25
local Module = {}

-----------------------------------------
local _
local tinsert						= table.insert
local UnitCanAttack					= UnitCanAttack
local UnitDetailedThreatSituation	= UnitDetailedThreatSituation
-----------------------------------------

Module.Frames = {}

local function UpdateElement(Element, elapsed)
	if Element.Disabled or not UnitExists(Element.Unit) then return end
	
	Element.lastUpdate = (Element.lastUpdate or 0) + elapsed
	if Element.lastUpdate > UpdateInterval then
		Element:SetValue(select(3, UnitDetailedThreatSituation("player", Element.Unit)) or 0)
		
		Element.lastUpdate = 0
	end
end

local function OnEvent(Element, event, ...)
	if Element.Disabled or not UnitExists(Element.Unit) or Element.Unit ~= ... then return end
	
	if UnitCanAttack("player", Element.Unit) then
		Element:Show()
	else
		Element:Hide()
	end
	
	UpdateElement(Element, UpdateInterval)
end

local function ForceUpdate(Element)
	OnEvent(Element, 'ForceUpdate', Element.Unit)
	UpdateElement(Element, UpdateInterval)
end

----------

function Module:LoadProfile()
	local ProfileTarget
	for _, F in pairs(self.Frames) do
		ProfileTarget = self.db.units[F.ProfileUnit]
		
		F.Threat:UnregisterAllEvents()
		
		if not ProfileTarget.threat.enable then
			F.Threat.Disabled = true
		else	
			F.Threat.Disabled = false
			
			F.Threat:RegisterEvent("UNIT_FACTION")
			F.Threat:SetSize(5, ProfileTarget.health.height)
			F.Threat:ClearAllPoints()
			F.Threat:SetPoint('LEFT', F, 'RIGHT')
		end
		
		OnEvent(F.Threat)
	end
end

function Module:Create(F)
	local Element = E:CreateBar("CUI_ThreatBar", "LOW", 5, 68, {"LEFT", F, "RIGHT"}, F, 0, 0, 0)
	F.Threat = Element
	
	Element:SetMinMaxValues(0, 100)
	Element:SetValue(0)
	
	Element.Overlay:SetOrientation("VERTICAL")
	Element.Overlay:GetStatusBarTexture():SetVertexColor(1, 0.8, 0.17) -- Yellow
	
	Element.Unit = F.Unit
	
	F.Threat:SetScript("OnUpdate", UpdateElement)
	Element:SetScript("OnEvent", OnEvent)
	
	Element.ForceUpdate = ForceUpdate
	tinsert(self.Frames, F)
end

---------- Add Module
UF:RegisterModule('Threat', Module)